SystemOrganization addCategory: #'CTM-Core'!
SystemOrganization addCategory: #'CTM-Examples'!
SystemOrganization addCategory: #'CTM-Tests'!

!MethodContext methodsFor: '*CTM-Core-Override' stamp: 'mig 1/30/2011 21:49'!
printString
	"Answer an emphasized string in case of a breakpoint method, and an italic string for generated methods."
	
	| printText |
	printText := super printString asText.
	
	(self method notNil and: [self method hasBreakpoint]) ifTrue: [
		printText append: ' [break]'.
		printText allBold].
	
	(self method notNil and: [self method isCTMGeneratedMethod]) ifTrue: [
		printText append: ' [generated]'.
		printText addAttribute: TextEmphasis italic].
	
	^ printText
! !

!ClassDescription methodsFor: '*CTM-Core-override' stamp: 'SR 1/31/2011 12:06'!
addAndClassifySelector: selector withMethod: compiledMethod inProtocol: category notifying: requestor
	| priorMethodOrNil |
	priorMethodOrNil := self compiledMethodAt: selector ifAbsent: [nil].
	
	"New system event"
	[ SystemChangeNotifier uniqueInstance methodCompiled: compiledMethod selector: selector inProtocol: category class: self requestor: requestor ]
	on: Exception do: [ :e | e signal. ^ nil ].
	
	self addSelectorSilently: selector withMethod: compiledMethod.
	SystemChangeNotifier uniqueInstance 
		doSilently: [self organization classify: selector under: category].
	priorMethodOrNil isNil
		ifTrue: [SystemChangeNotifier uniqueInstance methodAdded: compiledMethod selector: selector inProtocol: category class: self requestor: requestor]
		ifFalse: [SystemChangeNotifier uniqueInstance methodChangedFrom: priorMethodOrNil to: compiledMethod selector: selector inClass: self requestor: requestor].! !

!ClassDescription methodsFor: '*CTM-Core-override' stamp: 'SR 1/31/2011 12:06'!
addSelector: selector withMethod: compiledMethod notifying: requestor
	| priorMethodOrNil |
	priorMethodOrNil := self compiledMethodAt: selector ifAbsent: [nil].

	"New system event"
	[ SystemChangeNotifier uniqueInstance methodCompiled: compiledMethod selector: selector class: self requestor: requestor ]
	on: Exception do: [ :e | e signal. ^ nil ].
	
	self addSelectorSilently: selector withMethod: compiledMethod.
	priorMethodOrNil isNil
		ifTrue: [SystemChangeNotifier uniqueInstance methodAdded: compiledMethod selector: selector inClass: self requestor: requestor]
		ifFalse: [SystemChangeNotifier uniqueInstance methodChangedFrom: priorMethodOrNil to: compiledMethod selector: selector inClass: self requestor: requestor].! !

!ClassDescription methodsFor: '*CTM-Core-override' stamp: 'mig 1/7/2011 16:23'!
browserIcon: aClassDescription selector: aSelector
	"Answer an icon to be shown for the selector aSymbol in aClassDescription."

	aClassDescription methodDictionary at: aSelector ifPresent: [ :method |
		method literalsDo: [ :literal |
			(literal == #halt or: [ literal == #halt: or: [ literal == #haltIfNil or: [ literal == #haltIf: or: [ literal == #haltOnce or: [ literal == #haltOnce: or: [ literal == #haltOnCount: or: [ literal == #halt:onCount: ] ] ] ] ] ] ])
				ifTrue: [ ^ #breakpoint ].
			(literal == #flag: or: [ literal == #needsWork or: [ literal == #notYetImplemented ] ])
				ifTrue: [ ^ #flag ] ] ].
	(aClassDescription methodDictionary at: aSelector) isCTMGeneratedMethod ifTrue: [^ #generated].
	^ (aClassDescription isOverride: aSelector)
		ifTrue: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ #arrowUpAndDown ]
				ifFalse: [ #arrowUp ] ]
		ifFalse: [
			(aClassDescription isOverridden: aSelector)
				ifTrue: [ #arrowDown ]
				ifFalse: [ #blank ] ]! !

!Parser methodsFor: '*CTM-Override' stamp: 'mig 1/14/2011 14:39'!
method: doit context: ctxt 
	" pattern [ | temporaries ] block => MethodNode."

	| selector arguments precedence blk prim temps messageComment methodNode |
	(self pattern: doit inContext: ctxt) in: [:sap |
		selector := sap first.
		arguments := sap second.
		precedence := sap third.].
	self properties selector: selector.
	encoder selector: selector.
	arguments do: [:argNode | 
		argNode beMethodArg].
	self pragmaSequence.
	temps := self temporaries.
	messageComment := currentComment.
	currentComment := nil.	
	self pragmaSequence.
	prim := self pragmaPrimitives.
	self statements: #() innerBlock: doit.
	blk := parseNode.
	doit
		ifTrue: [blk returnLast]
		ifFalse: [blk returnSelfIfNoOther: encoder].
	hereType == #doIt ifFalse: [^self expected: 'Nothing more'].
	methodNode := self newMethodNode.
	methodNode
		comment: messageComment;
		selector: selector
		arguments: arguments
		precedence: precedence
		temporaries: temps
		block: blk
		encoder: encoder
		primitive: prim
		properties: properties.
	self interactive ifTrue:
		[self declareUndeclaredTemps: methodNode.
		 self removeUnusedTemps].
	^methodNode
! !

!Parser methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 19:59'!
parsePragma: aString

	| stream |
	stream := ReadStream on: aString.
	self init: stream notifying: nil failBlock: [^ false].
	self pragmaSequence.
	^ self properties pragmas first
	

! !

!Pragma class methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 20:27'!
fromString: aString

	^ Parser new parsePragma: aString! !

!Metaclass methodsFor: '*CTM-Core' stamp: 'mig 1/15/2011 12:25'!
pragmaAt: aKey

	^ self pragmas detect: [:pragma | pragma key = aKey] ifNone: [nil]! !

!Metaclass methodsFor: '*CTM-Core' stamp: 'mig 1/15/2011 12:21'!
pragmas

	^ pragmas ifNil: [pragmas := OrderedCollection new]! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'mig 1/26/2011 23:42'!
ctmGeneratedMethodsForThis

	^ self propertyValueAt: #ctmGeneratedMethods ifAbsent: [Set new]! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'mig 1/21/2011 16:50'!
ctmGeneratedMethodsForThis: aCollection

	^ self propertyValueAt: #ctmGeneratedMethods put: aCollection! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'mig 1/26/2011 19:43'!
ctmGeneratingMethod

	^ self propertyValueAt: #ctmGeneratingMethod ifAbsent: nil! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'mig 1/26/2011 19:44'!
ctmGeneratingMethod: aCompiledMethod

	^ self propertyValueAt: #ctmGeneratingMethod put: aCompiledMethod! !

!CompiledMethod methodsFor: '*CTM-Examples' stamp: 'SR 1/14/2011 16:32'!
hasInstVarStore
	"Answer whether the method references an instance variable."

	| scanner end printer |

	scanner := InstructionStream on: self.
	printer := CTMInstVarStoreLocator new.
	end := self endPC.

	[scanner pc <= end] whileTrue: [
		(printer interpretNextInstructionUsing: scanner) ifTrue: [^true].
	].
	^false! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'mig 1/26/2011 23:05'!
isCTMGeneratedMethod

	^ self ctmGeneratingMethod notNil! !

!CompiledMethod methodsFor: '*CTM-Core' stamp: 'SR 1/7/2011 13:56'!
selectorParts

	^ (self selector asString subStrings: ':') collect: [:each | (each,':') asSymbol]! !

!Class methodsFor: '*CTM-Core' stamp: 'SR 1/22/2011 12:09'!
addInstVarName: aString after: anInstVarName
	"Add the argument, aString, as one of the receiver's instance variables."
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: (instanceVariables inject: '' into: [:sum :part | (part = anInstVarName) ifTrue: [ sum,' ',part,' ',aString] ifFalse: [sum, ' ', part]])
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category
! !

!Class methodsFor: '*CTM-Core-override' stamp: 'SR 1/21/2011 16:19'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		traitComposition: self traitCompositionString
		classTraitComposition: self class traitCompositionString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	
		pragmas: self pragmas! !

!Class methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 13:29'!
definition

	^ (self theMetaClass pragmas inject: '' into: [:sum :each | sum, each printString, String cr]), super definition! !

!Class methodsFor: '*CTM-Core' stamp: 'mig 1/14/2011 16:20'!
pragmaAt: aKey

	^ self theMetaClass pragmaAt: aKey! !

!Class methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 13:27'!
pragmas

	^ self superclass isNil
		ifTrue:  [self theMetaClass pragmas]
		ifFalse: [(self theMetaClass pragmas) copy addAll: self superclass pragmas; yourself]! !

!SystemChangeNotifier methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 15:00'!
classCompiled: aClass category: aCategory
	"A method with the given selector was added to aClass in protocol aCategoryName."

	self trigger: (CompiledEvent
				class: aClass
				category: aCategory)! !

!SystemChangeNotifier methodsFor: '*CTM-Core' stamp: 'SR 1/15/2011 21:21'!
methodCompiled: aMethod selector: aSymbol class: aClass requestor: requestor
	"A method with the given selector was added to aClass in protocol aCategoryName."

	self trigger: (CompiledEvent
				method: aMethod
				selector: aSymbol
				class: aClass
				requestor: requestor)! !

!SystemChangeNotifier methodsFor: '*CTM-Core' stamp: 'SR 1/15/2011 21:21'!
methodCompiled: aMethod selector: aSymbol inProtocol: aCategoryName class: aClass requestor: requestor
	"A method with the given selector was added to aClass in protocol aCategoryName."

	self trigger: (CompiledEvent
				method: aMethod
				selector: aSymbol
				protocol: aCategoryName
				class: aClass
				requestor: requestor)! !

!Association methodsFor: '*CTM-Core' stamp: 'SR 1/14/2011 14:20'!
analogousCodeTo: anObject 

	^ self = anObject! !

!DataStream methodsFor: '*CTM-Core' stamp: 'SR 1/22/2011 11:02'!
insideASegment
	^ false! !

!ClassBuilder methodsFor: '*CTM-Core-override' stamp: 'SR 1/22/2011 11:07'!
name: className inEnvironment: env subclassOf: newSuper type: type instanceVariableNames: instVarString classVariableNames: classVarString poolDictionaries: poolString category: category unsafe: unsafe
	"Define a new class in the given environment.
	If unsafe is true do not run any validation checks.
	This facility is provided to implement important system changes."
	| pragmas |

	"Add class pragmas"
	(thisContext findContextSuchThat: [:con | con method selector isDoIt]) ifNotNilDo: [:con | 
		pragmas := con method pragmas collect: [:oldp | Pragma keyword: (oldp keyword) arguments: (oldp arguments)]].
	
	^ self name: className inEnvironment: env subclassOf: newSuper type: type instanceVariableNames: instVarString classVariableNames: classVarString poolDictionaries: poolString category: category unsafe: unsafe pragmas: pragmas.! !

!ClassBuilder methodsFor: '*CTM-Core' stamp: 'SR 1/31/2011 12:06'!
name: className inEnvironment: env subclassOf: newSuper type: type instanceVariableNames: instVarString classVariableNames: classVarString poolDictionaries: poolString category: category unsafe: unsafe pragmas: pragmas
	"Define a new class in the given environment.
	If unsafe is true do not run any validation checks.
	This facility is provided to implement important system changes."
	| oldClass instVars classVars copyOfOldClass newClass |
 
	environ := env.
	instVars := Scanner new scanFieldNames: instVarString.
	classVars := (Scanner new scanFieldNames: classVarString) collect: [:x | x asSymbol].

	"Validate the proposed name"
	unsafe ifFalse:[(self validateClassName: className) ifFalse:[^nil]].
	oldClass := env at: className ifAbsent:[nil].
	oldClass isBehavior 
		ifFalse: [oldClass := nil]  "Already checked in #validateClassName:"
		ifTrue: [
			copyOfOldClass := oldClass copy.
			copyOfOldClass superclass addSubclass: copyOfOldClass].
	
	
	[ | newCategory needNew force organization oldCategory |
	unsafe ifFalse:[
		"Run validation checks so we know that we have a good chance for recompilation"
		(self validateSuperclass: newSuper forSubclass: oldClass) ifFalse:[^nil].
		(self validateInstvars: instVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateClassvars: classVars from: oldClass forSuper: newSuper) ifFalse:[^nil].
		(self validateSubclassFormat: type from: oldClass forSuper: newSuper extra: instVars size) ifFalse:[^nil]].

	"See if we need a new subclass"
	needNew := self needsSubclassOf: newSuper type: type instanceVariables: instVars from: oldClass.
	needNew == nil ifTrue:[^nil]. "some error"

	(needNew and:[unsafe not]) ifTrue:[
		"Make sure we don't redefine any dangerous classes"
		(self tooDangerousClasses includes: oldClass name) ifTrue:[
			self error: oldClass name, ' cannot be changed'.
		].
		"Check if the receiver should not be redefined"
		(oldClass ~~ nil and:[oldClass shouldNotBeRedefined]) ifTrue:[
			self notify: oldClass name asText allBold, 
						' should not be redefined. \Proceed to store over it.' withCRs]].

	needNew ifTrue:[
		"Create the new class"
		newClass := self 
			newSubclassOf: newSuper 
			type: type 
			instanceVariables: instVars
			from: oldClass.
		newClass == nil ifTrue:[^nil]. "Some error"
		newClass setName: className.
	] ifFalse:[
		"Reuse the old class"
		newClass := oldClass.
	].

	"Add class pragmas"
	newClass theMetaClass pragmas removeAll.
	pragmas ifNotNil: [newClass theMetaClass pragmas addAll: pragmas].
	
	[ SystemChangeNotifier uniqueInstance classCompiled: newClass category: category ]
	on: Exception do: [ :e | e signal. ^ nil ].

	"Install the class variables and pool dictionaries... "
	force := (newClass declare: classVarString) | (newClass sharing: poolString).

	"... classify ..."
	newCategory := category asSymbol.
	organization := environ ifNotNil:[environ organization].
	oldClass isNil ifFalse: [oldCategory := (organization categoryOfElement: oldClass name) asSymbol].
	organization classify: newClass name under: newCategory.
	newClass environment: environ.

	"... recompile ..."
	newClass := self recompile: force from: oldClass to: newClass mutate: false.

	"... export if not yet done ..."
	(environ at: newClass name ifAbsent:[nil]) == newClass ifFalse:[
		[environ at: newClass name put: newClass]
			on: AttemptToWriteReadOnlyGlobal do:[:ex| ex resume: true].
		environ flushClassNameCache.
	].


	newClass doneCompiling.
	"... notify interested clients ..."
	oldClass isNil ifTrue: [
		SystemChangeNotifier uniqueInstance classAdded: newClass inCategory: newCategory.
		^ newClass].
	newCategory ~= oldCategory 
		ifTrue: [SystemChangeNotifier uniqueInstance class: newClass recategorizedFrom: oldCategory to: category]
		ifFalse: [SystemChangeNotifier uniqueInstance classDefinitionChangedFrom: copyOfOldClass to: newClass.].
] ensure: 
		[copyOfOldClass ifNotNil: [copyOfOldClass superclass removeSubclass: copyOfOldClass].
		Behavior flushObsoleteSubclasses.
		].
	^newClass! !

!MCPackage methodsFor: '*CTM-Core' stamp: 'SR 1/7/2011 15:00'!
snapshot
	| packageInfo definitions categories |
	packageInfo := self packageInfo.
	definitions := OrderedCollection new.
	categories := packageInfo systemCategories.
	categories isEmpty ifFalse: [ definitions add: (MCOrganizationDefinition categories: categories) ].
	packageInfo methods do: [:ea | ea isReferenceToGeneratedMethod ifFalse: [ definitions add: ea asMethodDefinition]] displayingProgress: 'Snapshotting methods...'.
	(packageInfo respondsTo: #overriddenMethods) ifTrue:
		[packageInfo overriddenMethods
			do: [:ea | definitions add:
					(packageInfo changeRecordForOverriddenMethod: ea) asMethodDefinition]
			displayingProgress: 'Searching for overrides...'].
	packageInfo classes do: [:ea | definitions addAll: ea classDefinitions] displayingProgress: 'Snapshotting classes...'.
	(packageInfo respondsTo: #hasPreamble) ifTrue: [
		packageInfo hasPreamble ifTrue: [definitions add: (MCPreambleDefinition from: packageInfo)].
		packageInfo hasPostscript ifTrue: [definitions add: (MCPostscriptDefinition from: packageInfo)].
		packageInfo hasPreambleOfRemoval ifTrue: [definitions add: (MCRemovalPreambleDefinition from: packageInfo)].
		packageInfo hasPostscriptOfRemoval ifTrue: [definitions add: (MCRemovalPostscriptDefinition from: packageInfo)]]. 
	^ MCSnapshot fromDefinitions: definitions
! !

!Debugger methodsFor: '*CTM-Core' stamp: 'mig 1/31/2011 11:45'!
ctmSkipOutOfGenerated

	"don't do anything if shift is pressed"
	Sensor shiftPressed ifTrue: [^self].
	
	"skip over all generated methods"
	[interruptedProcess suspendedContext method isCTMGeneratedMethod] whileTrue: [
		| target sender |
		target := interruptedProcess suspendedContext method ctmGeneratingMethod.
		sender := interruptedProcess suspendedContext sender.
		
		[(interruptedProcess suspendedContext method = target) or: [interruptedProcess suspendedContext = sender]] whileFalse: [
			interruptedProcess stepToSendOrReturn; step]].
! !

!Debugger methodsFor: '*CTM-Core-Override' stamp: 'mig 1/30/2011 21:32'!
send
	"Send the selected message in the accessed method, and take control in 
	the method invoked to allow further step or send."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	interruptedProcess step: self selectedContext.
	self ctmSkipOutOfGenerated.
	self resetContext: interruptedProcess stepToSendOrReturn.
! !

Object subclass: #CTMEventListener
	instanceVariableNames: 'methodManagement classManagement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Core'!

CTMEventListener class
	instanceVariableNames: 'instance'!

CTMEventListener class
	instanceVariableNames: 'instance'!

!CTMEventListener class methodsFor: 'singleton handling' stamp: 'mig 12/8/2010 20:35'!
instance

	^ instance ifNil: [ instance := self new ]! !

!CTMEventListener class methodsFor: 'singleton handling' stamp: 'mig 1/7/2011 10:37'!
reset

	instance ifNotNil: [instance unregister].
	instance := nil.
! !

!CTMEventListener methodsFor: 'private' stamp: 'mig 2/1/2011 11:43'!
allMacros

	^ CTMMacro allSubclasses reject: [:macro | macro isAbstract ]! !

!CTMEventListener methodsFor: 'event-processing' stamp: 'mig 2/1/2011 11:45'!
classEvent: anEvent

	"`anEvent item` is the class object"
	anEvent item pragmas do: [ :pragma |
		self allMacros do: [:macro |
			macro new in: [ :macroInstance | 
				(macroInstance matchClassPragma: pragma) ifTrue: [
					macroInstance handleClassEvent: anEvent]]]].! !

!CTMEventListener methodsFor: 'event-processing' stamp: 'SR 1/14/2011 14:03'!
event: anEvent

	(anEvent itemKind == #method) ifTrue: [ self methodEvent: anEvent ].
	(anEvent itemKind == #class) ifTrue: [ self classEvent: anEvent ].! !

!CTMEventListener methodsFor: 'event-processing' stamp: 'mig 2/1/2011 18:50'!
methodEvent: anEvent

	| method |
	method := self methodManagement prepareMethod: anEvent item from: anEvent oldItem.
		
	"Handle pragmas of this method and the enclosing class"
	(method pragmas, method methodClass pragmas) do: [ :pragma |
		self allMacros do: [:macro |
			macro new in: [ :macroInstance | 
				(macroInstance matchMethodPragma: pragma) ifTrue: [
					macroInstance handleMethodEvent: anEvent]]]].
		
	"If changed class is a CTMMacro, then recompile all known users of this CTMMacro"
	anEvent isCompiled ifFalse: [
		method methodClass isCTMMacro ifTrue: [
			method methodClass associatedMethods 
				do: [ :usingMethod |	methodManagement recompile: usingMethod from: usingMethod methodClass]
				displayingProgress: 'Recompiling macro users...']].! !

!CTMEventListener methodsFor: 'private' stamp: 'SR 1/7/2011 11:53'!
methodManagement

	^ methodManagement ifNil: [ methodManagement := CTMMethodManagement new]! !

!CTMEventListener methodsFor: 'system-integration' stamp: 'mig 12/9/2010 20:12'!
register

	SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:! !

!CTMEventListener methodsFor: 'system-integration' stamp: 'mig 12/9/2010 20:13'!
unregister

	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self! !

Object subclass: #CTMMacro
	instanceVariableNames: 'methodManagement classManagement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Core'!

CTMMacro class
	instanceVariableNames: 'associatedMethods associatedClasses'!

CTMMacro class
	instanceVariableNames: 'associatedMethods associatedClasses'!

CTMMacro subclass: #CTMImmutableMacro
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Examples'!

!CTMImmutableMacro class methodsFor: 'configuration' stamp: 'SR 1/14/2011 16:36'!
specialSelectors
	"Returns an array of selectors, which can violate the immutable constraint"
	
	^ {#initialize.}
	! !

!CTMImmutableMacro methodsFor: 'class-events' stamp: 'mig 2/1/2011 19:51'!
classCompiled: aClass

	"check whether superclass is immutable or has no instance variables, throw error if not"
	(aClass superclass instSize isZero or: [aClass superclass pragmas anySatisfy: [ :pragma | self matchClassPragma: pragma]]) ifTrue: [ ^ self ].
	
	Error signal: 'The superclass needs to be immutable too.'! !

!CTMImmutableMacro methodsFor: 'pragma-matching' stamp: 'SR 1/14/2011 16:38'!
matchClassPragma: aPragma

	(aPragma keyword = #immutable) ifTrue: [
		^ true].
	
	^ super matchMethodPragma: aPragma! !

!CTMImmutableMacro methodsFor: 'pragma-matching' stamp: 'SR 1/14/2011 16:38'!
matchMethodPragma: aPragma

	(aPragma keyword = #immutable) ifTrue: [
		^ true].
	
	^ super matchMethodPragma: aPragma! !

!CTMImmutableMacro methodsFor: 'method-events' stamp: 'mig 2/1/2011 18:25'!
methodCompiled: aMethod in: aClass
	
	"check whether the method has a special name or is flagged as initializer"
	((self class specialSelectors includes: (aMethod selector))
		or: [(aMethod pragmaAt: #initializer) notNil]) ifFalse: [
			"raise an error if the method contains an instance variable store opcode"
			aMethod hasInstVarStore ifTrue: [
				Error signal: 'This class is immutable']].! !

!CTMMacro class methodsFor: 'accessing' stamp: 'SR 1/14/2011 15:44'!
associatedClasses

	^ associatedClasses ifNil: [ associatedClasses := OrderedCollection new]! !

!CTMMacro class methodsFor: 'accessing' stamp: 'SR 1/14/2011 11:51'!
associatedMethods

	^ associatedMethods ifNil: [ associatedMethods := OrderedCollection new]! !

!CTMMacro class methodsFor: 'configuration' stamp: 'mig 1/26/2011 23:31'!
convertModifiedToAddedRemoved

	"should `modified` events be converted into separate add/remove events?"
	^ true! !

!CTMMacro class methodsFor: 'testing' stamp: 'mig 1/26/2011 23:31'!
isAbstract

	"describes whether this is an abstract implementation that should not be instantiated, similiar to TestCase"
	^ false! !

!CTMMacro class methodsFor: 'testing' stamp: 'mig 2/1/2011 11:41'!
isCTMMacro

	^ true! !

!CTMMacro methodsFor: 'class-events' stamp: 'SR 1/14/2011 15:38'!
classAdded: aClass! !

!CTMMacro methodsFor: 'class-events' stamp: 'SR 1/21/2011 15:02'!
classCompiled: aClass! !

!CTMMacro methodsFor: 'class-events' stamp: 'SR 1/14/2011 15:38'!
classModified: aClass from: anOldClass

	self class convertModifiedToAddedRemoved ifTrue: [
		self classRemoved: anOldClass.
		self classAdded: aClass]! !

!CTMMacro methodsFor: 'class-events' stamp: 'SR 1/14/2011 15:38'!
classRemoved: aClass! !

!CTMMacro methodsFor: 'internal' stamp: 'SR 1/21/2011 15:02'!
handleClassEvent: anEvent

	| class |
	class := anEvent item.

	anEvent isAdded ifTrue: [ ^ self classAdded: class ].
	anEvent isModified ifTrue: [ ^ self classModified: class from: (anEvent oldItem) ].
	anEvent isCompiled ifTrue: [ ^ self classCompiled: class ].
	anEvent isRemoved ifTrue: [ ^ self classRemoved: class ].! !

!CTMMacro methodsFor: 'internal' stamp: 'SR 1/15/2011 21:24'!
handleMethodEvent: anEvent

	| class method |
	method := anEvent item.
	class := anEvent environmentAt: #class.

	anEvent isAdded ifTrue: [ ^ self methodAdded: method in: class ].
	anEvent isModified ifTrue: [ ^ self methodModified: method in: class from: (anEvent oldItem) ].
	anEvent isCompiled ifTrue: [ ^ self methodCompiled: method in: class ].
	anEvent isRemoved ifTrue: [ ^ self methodRemoved: method in: class ].! !

!CTMMacro methodsFor: 'pragma-matching' stamp: 'SR 1/6/2011 22:03'!
matchClassPragma: aPragma

	^ false! !

!CTMMacro methodsFor: 'pragma-matching' stamp: 'SR 1/6/2011 21:25'!
matchMethodPragma: aPragma

	^ false! !

!CTMMacro methodsFor: 'method-events' stamp: 'SR 1/14/2011 12:06'!
methodAdded: aMethod in: aClass

	self class associatedMethods add: aMethod! !

!CTMMacro methodsFor: 'method-events' stamp: 'SR 1/15/2011 21:24'!
methodCompiled: aCompiledMethod in: aClass! !

!CTMMacro methodsFor: 'accessing' stamp: 'SR 1/6/2011 22:03'!
methodManagement

	^ methodManagement ifNil: [ methodManagement := CTMMethodManagement new].! !

!CTMMacro methodsFor: 'method-events' stamp: 'SR 1/6/2011 22:00'!
methodModified: aMethod in: aClass from: anOldMethod

	self class convertModifiedToAddedRemoved ifTrue: [
		self methodRemoved: anOldMethod in: aClass.
		self methodAdded: aMethod in: aClass]! !

!CTMMacro methodsFor: 'method-events' stamp: 'SR 1/14/2011 14:18'!
methodRemoved: aMethod in: aClass

	self class associatedMethods remove: aMethod ifAbsent: [].! !

CTMMacro subclass: #CTMOptionalParameterMacro
	instanceVariableNames: 'config'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Examples'!

!CTMOptionalParameterMacro methodsFor: 'pragma-matching' stamp: 'SR 1/7/2011 13:37'!
config

	^ config ifNil: [ config := Dictionary new].! !

!CTMOptionalParameterMacro methodsFor: 'pragma-matching' stamp: 'mig 1/7/2011 14:38'!
matchMethodPragma: aPragma

	(aPragma keyword asString matchesRegex: '^(optional\:default\:)*$') ifTrue: [
		aPragma arguments pairsDo: [:key :value |
			self config at: key put: value].
		^ true].
	
	^ super matchMethodPragma: aPragma! !

!CTMOptionalParameterMacro methodsFor: 'method-events' stamp: 'mig 2/1/2011 18:43'!
methodAdded: aMethod in: aClass

	super methodAdded: aMethod in: aClass.

	self config keys allCombinationsDo: [:each | | signature actualArguments arguments code |
		actualArguments := aMethod selectorParts difference: each.
		actualArguments ifNotEmpty: [
			signature := actualArguments collectWithIndex: [ :each2 :index | '{1} t{2} ' format: {each2 asString. index asString.}].

			arguments := aMethod selectorParts collect: [:each2 | (actualArguments includes: each2)
					ifFalse: [' <1s> <2p>' expandMacrosWith: each2 with: (self config at: each2)]
					ifTrue: [' <1s> t<2p>' expandMacrosWith: each2 with: (actualArguments indexOf: each2)]].
			
			code := '<1s><n><n><t>^ self<2s>' expandMacrosWith: signature concatenation with: arguments concatenation.
			self methodManagement addGeneratedMethodFor: aMethod with: code.
		].
	].! !

CTMMacro subclass: #CTMSealedClassMacro
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Examples'!

!CTMSealedClassMacro methodsFor: 'as yet unclassified' stamp: 'SR 1/21/2011 15:11'!
classCompiled: aClass

	(aClass superclass pragmas anySatisfy: [ :pragma | pragma keyword = #sealed]) ifTrue: [ 
		Error signal: 'The superclass is marked as sealed.'].
	
! !

!CTMSealedClassMacro methodsFor: 'as yet unclassified' stamp: 'SR 1/21/2011 15:11'!
matchClassPragma: aPragma

	^ true! !

CTMMacro subclass: #CTMShortcutsMacro
	instanceVariableNames: 'shortcutName shortcutMaxCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Examples'!

!CTMShortcutsMacro methodsFor: 'pragma-matching' stamp: 'SR 1/6/2011 21:50'!
matchMethodPragma: aPragma

	(aPragma keyword = #shortcuts:count:) ifTrue: [
		shortcutName := aPragma argumentAt: 1.
		shortcutMaxCount := aPragma argumentAt: 2.
		^ true].
	
	^ super matchMethodPragma: aPragma! !

!CTMShortcutsMacro methodsFor: 'method-events' stamp: 'mig 2/1/2011 18:47'!
methodAdded: aMethod in: aClass
	
	| methodName collectionCreation parameterName |
	super methodAdded: aMethod in: aClass.
	
	methodName := ''.
	collectionCreation := ''.
	parameterName := aMethod parseTree arguments last token value, 'Element'.
	
	1 to: shortcutMaxCount do: [ :i | | code |
		methodName := '<1s> <2s>: <3s><4p> ' expandMacrosWith: methodName with: shortcutName with: parameterName with: i.
		collectionCreation := '<1s><2s><3p>. ' expandMacrosWith: collectionCreation with: parameterName with: i.
		
		code := '<1s><n><t>^ self <2s> {<3s>}' expandMacrosWith: methodName with: aMethod selector with: collectionCreation.
		self methodManagement addGeneratedMethodFor: aMethod with: code.
	].! !

CTMMacro subclass: #CTMTestMacro
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Tests'!

!CTMTestMacro methodsFor: 'pragma-matching' stamp: 'SR 1/15/2011 21:14'!
matchMethodPragma: aPragma

	(aPragma keyword = #ctmTest) ifTrue: [
		^ true].
	
	^ super matchMethodPragma: aPragma! !

Object subclass: #CTMMethodManagement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Core'!

!CTMMethodManagement methodsFor: 'generated-method-handling' stamp: 'mig 1/26/2011 23:44'!
addGeneratedMethodFor: aCompiledMethod with: aCodeString

	"Add a generated using aCodeString; the generating method is aCompiledMethod"

	| class method generatedMethods |
	class := aCompiledMethod methodClass.
	method := self addMethodTo: class in: #generated with: aCodeString.
	method ifNil: [ ^ nil ].
	
	"Mark new method as generated method"
	method ctmGeneratingMethod: aCompiledMethod.
	
	"Add new method to generated methods list"
	
	generatedMethods := aCompiledMethod ctmGeneratedMethodsForThis.
	generatedMethods add: (method selector).
	aCompiledMethod ctmGeneratedMethodsForThis: generatedMethods.! !

!CTMMethodManagement methodsFor: 'normal-method-handling' stamp: 'mig 1/26/2011 23:13'!
addMethodTo: aClass in: aCategorySymbol with: aCodeString

	| methodAndNode selector |
	methodAndNode := aClass 
		compile: aCodeString 
		classified: aCategorySymbol 
		notifying: nil 
		trailer: aClass 
		defaultMethodTrailer 
		ifFail: [^nil].
	selector := methodAndNode selector.
	
	(aClass methodDictionary includesKey: selector) ifTrue: [
		"Don't overwrite user-written methods."
		(aClass>>selector) isCTMGeneratedMethod ifFalse: [ ^ nil ].
	].
	
	aClass
		logMethodSource: aCodeString forMethodWithNode: methodAndNode inCategory: aCategorySymbol withStamp: (Utilities changeStamp) notifying: nil;
		addAndClassifySelector: selector withMethod: methodAndNode method inProtocol: aCategorySymbol notifying: nil.
		
	^ methodAndNode method! !

!CTMMethodManagement methodsFor: 'method-preparation' stamp: 'mig 1/26/2011 23:14'!
copyCTMPropertiesFrom: anOldCompiledMethod to: aCompiledMethod
	
	"Copy CTM-relevant properties from one CompiledMethod to another."
	aCompiledMethod 
		ctmGeneratingMethod: anOldCompiledMethod ctmGeneratingMethod;
		ctmGeneratedMethodsForThis: anOldCompiledMethod ctmGeneratedMethodsForThis.! !

!CTMMethodManagement methodsFor: 'method-preparation' stamp: 'mig 1/26/2011 23:14'!
prepareMethod: aCompiledMethod from: anOldCompiledMethod

	"Copy our properties from the old method to the new one"
	anOldCompiledMethod ifNotNil: [self copyCTMPropertiesFrom: anOldCompiledMethod to: aCompiledMethod].
	
	"Cleanup generated methods"
	self removeAllGeneratedMethodsFor: aCompiledMethod.
	
	^ aCompiledMethod methodClass compiledMethodAt: (aCompiledMethod selector) ifAbsent: [
		"The method is not in the method dict, so this was caused by an remove event"
		aCompiledMethod ]! !

!CTMMethodManagement methodsFor: 'private' stamp: 'mig 1/26/2011 23:17'!
recompile: aMethod from: aClass

	aClass compile: aMethod getSource notifying: nil.! !

!CTMMethodManagement methodsFor: 'generated-method-handling' stamp: 'SR 1/7/2011 13:16'!
removeAllGeneratedMethodsFor: aCompiledMethod

	| class |
	class := aCompiledMethod methodClass.
	
	aCompiledMethod ctmGeneratedMethodsForThis do: [ :methodSelector |
		(class methodDictionary includesKey: methodSelector) ifTrue: [
			self removeMethod: (class>>methodSelector)]].

	aCompiledMethod ctmGeneratedMethodsForThis: (OrderedCollection new).! !

!CTMMethodManagement methodsFor: 'normal-method-handling' stamp: 'SR 1/7/2011 11:06'!
removeMethod: aCompiledMethod

	| class selector |
	class := aCompiledMethod methodClass.
	selector := aCompiledMethod selector.
	
	self removeAllGeneratedMethodsFor: aCompiledMethod.
	class removeSelector: selector.! !

Object subclass: #CTMSystemPreparation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Core'!

!CTMSystemPreparation commentStamp: 'mig 1/30/2011 18:52' prior: 0!
The only purpose of this class is to add some instance variables to present Squeak classes, which is not possible via Monticello right now.!

!CTMSystemPreparation class methodsFor: 'class initialization' stamp: 'mig 1/30/2011 18:48'!
initialize

	super initialize.
	
	"Add the `pragmas` instance variable to MetaClass"
	(Metaclass instVarNames includes: #pragmas) ifFalse: [
		Metaclass addInstVarName: #pragmas].
	
	"Add the `pragmas` instance variable to MCClassDefinition"
	(MCClassDefinition instVarNames includes: #pragmas) ifFalse: [
		MCClassDefinition addInstVarName: #pragmas after: 'commentStamp'].! !

Object subclass: #CTMTestSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Tests'!

!Object methodsFor: '*CTM-Core' stamp: 'mig 2/1/2011 11:42'!
isCTMMacro

	^ false! !

!OBMorphicIcons methodsFor: '*CTM-Core' stamp: 'mig 1/7/2011 16:17'!
generated
	^ ((ColorForm
	extent: 12@12
	depth: 8
	fromArray: #(0 0 0 0 0 0 0 0 0 0 3132733881 3103784960 186 3237148907 3233349632 185 4075151077 3854106624 185 4076863487 3736666112 185 3957251806 3736666112 186 3236224698 3115909120 0 3132733881 3103784960 0 0 0 0 0 0)
	offset: 0@0)
	colors: {Color transparent . Color black . Color white . Color gray . Color red . Color green . Color blue . Color cyan . Color yellow . Color magenta . Color veryVeryDarkGray . Color veryDarkGray . Color darkGray . Color lightGray . Color veryLightGray . Color veryVeryLightGray . (Color r: 0.031 g: 0.031 b: 0.031) . (Color r: 0.063 g: 0.063 b: 0.063) . (Color r: 0.094 g: 0.094 b: 0.094) . (Color r: 0.156 g: 0.156 b: 0.156) . (Color r: 0.188 g: 0.188 b: 0.188) . (Color r: 0.219 g: 0.219 b: 0.219) . (Color r: 0.282 g: 0.282 b: 0.282) . (Color r: 0.313 g: 0.313 b: 0.313) . (Color r: 0.344 g: 0.344 b: 0.344) . (Color r: 0.407 g: 0.407 b: 0.407) . (Color r: 0.438 g: 0.438 b: 0.438) . (Color r: 0.469 g: 0.469 b: 0.469) . (Color r: 0.531 g: 0.531 b: 0.531) . (Color r: 0.562 g: 0.562 b: 0.562) . (Color r: 0.593 g: 0.593 b: 0.593) . (Color r: 0.656 g: 0.656 b: 0.656) . (Color r: 0.687 g: 0.687 b: 0.687) . (Color r: 0.718 g: 0.718 b: 0.718) . (Color r: 0.781 g: 0.781 b: 0.781) . (Color r: 0.812 g: 0.812 b: 0.812) . (Color r: 0.844 g: 0.844 b: 0.844) . (Color r: 0.906 g: 0.906 b: 0.906) . (Color r: 0.937 g: 0.937 b: 0.937) . (Color r: 0.969 g: 0.969 b: 0.969) . Color black . (Color r: 0.0 g: 0.199 b: 0.0) . (Color r: 0.0 g: 0.4 b: 0.0) . (Color r: 0.0 g: 0.599 b: 0.0) . (Color r: 0.0 g: 0.8 b: 0.0) . Color green . (Color r: 0.0 g: 0.0 b: 0.199) . (Color r: 0.0 g: 0.199 b: 0.199) . (Color r: 0.0 g: 0.4 b: 0.199) . (Color r: 0.0 g: 0.599 b: 0.199) . (Color r: 0.0 g: 0.8 b: 0.199) . (Color r: 0.0 g: 1.0 b: 0.199) . (Color r: 0.0 g: 0.0 b: 0.4) . (Color r: 0.0 g: 0.199 b: 0.4) . (Color r: 0.0 g: 0.4 b: 0.4) . (Color r: 0.0 g: 0.599 b: 0.4) . (Color r: 0.0 g: 0.8 b: 0.4) . (Color r: 0.0 g: 1.0 b: 0.4) . (Color r: 0.0 g: 0.0 b: 0.599) . (Color r: 0.0 g: 0.199 b: 0.599) . (Color r: 0.0 g: 0.4 b: 0.599) . (Color r: 0.0 g: 0.599 b: 0.599) . (Color r: 0.0 g: 0.8 b: 0.599) . (Color r: 0.0 g: 1.0 b: 0.599) . (Color r: 0.0 g: 0.0 b: 0.8) . (Color r: 0.0 g: 0.199 b: 0.8) . (Color r: 0.0 g: 0.4 b: 0.8) . (Color r: 0.0 g: 0.599 b: 0.8) . (Color r: 0.0 g: 0.8 b: 0.8) . (Color r: 0.0 g: 1.0 b: 0.8) . Color blue . (Color r: 0.0 g: 0.199 b: 1.0) . (Color r: 0.0 g: 0.4 b: 1.0) . (Color r: 0.0 g: 0.599 b: 1.0) . (Color r: 0.0 g: 0.8 b: 1.0) . Color cyan . (Color r: 0.199 g: 0.0 b: 0.0) . (Color r: 0.199 g: 0.199 b: 0.0) . (Color r: 0.199 g: 0.4 b: 0.0) . (Color r: 0.199 g: 0.599 b: 0.0) . (Color r: 0.199 g: 0.8 b: 0.0) . (Color r: 0.199 g: 1.0 b: 0.0) . (Color r: 0.199 g: 0.0 b: 0.199) . (Color r: 0.199 g: 0.199 b: 0.199) . (Color r: 0.199 g: 0.4 b: 0.199) . (Color r: 0.199 g: 0.599 b: 0.199) . (Color r: 0.199 g: 0.8 b: 0.199) . (Color r: 0.199 g: 1.0 b: 0.199) . (Color r: 0.199 g: 0.0 b: 0.4) . (Color r: 0.199 g: 0.199 b: 0.4) . (Color r: 0.199 g: 0.4 b: 0.4) . (Color r: 0.199 g: 0.599 b: 0.4) . (Color r: 0.199 g: 0.8 b: 0.4) . (Color r: 0.199 g: 1.0 b: 0.4) . (Color r: 0.199 g: 0.0 b: 0.599) . (Color r: 0.199 g: 0.199 b: 0.599) . (Color r: 0.199 g: 0.4 b: 0.599) . (Color r: 0.199 g: 0.599 b: 0.599) . (Color r: 0.199 g: 0.8 b: 0.599) . (Color r: 0.199 g: 1.0 b: 0.599) . (Color r: 0.199 g: 0.0 b: 0.8) . (Color r: 0.199 g: 0.199 b: 0.8) . (Color r: 0.199 g: 0.4 b: 0.8) . (Color r: 0.199 g: 0.599 b: 0.8) . (Color r: 0.199 g: 0.8 b: 0.8) . (Color r: 0.199 g: 1.0 b: 0.8) . (Color r: 0.199 g: 0.0 b: 1.0) . (Color r: 0.199 g: 0.199 b: 1.0) . (Color r: 0.199 g: 0.4 b: 1.0) . (Color r: 0.199 g: 0.599 b: 1.0) . (Color r: 0.199 g: 0.8 b: 1.0) . (Color r: 0.199 g: 1.0 b: 1.0) . (Color r: 0.4 g: 0.0 b: 0.0) . (Color r: 0.4 g: 0.199 b: 0.0) . (Color r: 0.4 g: 0.4 b: 0.0) . (Color r: 0.4 g: 0.599 b: 0.0) . (Color r: 0.4 g: 0.8 b: 0.0) . (Color r: 0.4 g: 1.0 b: 0.0) . (Color r: 0.4 g: 0.0 b: 0.199) . (Color r: 0.4 g: 0.199 b: 0.199) . (Color r: 0.4 g: 0.4 b: 0.199) . (Color r: 0.4 g: 0.599 b: 0.199) . (Color r: 0.4 g: 0.8 b: 0.199) . (Color r: 0.4 g: 1.0 b: 0.199) . (Color r: 0.4 g: 0.0 b: 0.4) . (Color r: 0.4 g: 0.199 b: 0.4) . (Color r: 0.4 g: 0.4 b: 0.4) . (Color r: 0.4 g: 0.599 b: 0.4) . (Color r: 0.4 g: 0.8 b: 0.4) . (Color r: 0.4 g: 1.0 b: 0.4) . (Color r: 0.4 g: 0.0 b: 0.599) . (Color r: 0.4 g: 0.199 b: 0.599) . (Color r: 0.4 g: 0.4 b: 0.599) . (Color r: 0.4 g: 0.599 b: 0.599) . (Color r: 0.4 g: 0.8 b: 0.599) . (Color r: 0.4 g: 1.0 b: 0.599) . (Color r: 0.4 g: 0.0 b: 0.8) . (Color r: 0.4 g: 0.199 b: 0.8) . (Color r: 0.4 g: 0.4 b: 0.8) . (Color r: 0.4 g: 0.599 b: 0.8) . (Color r: 0.4 g: 0.8 b: 0.8) . (Color r: 0.4 g: 1.0 b: 0.8) . (Color r: 0.4 g: 0.0 b: 1.0) . (Color r: 0.4 g: 0.199 b: 1.0) . (Color r: 0.4 g: 0.4 b: 1.0) . (Color r: 0.4 g: 0.599 b: 1.0) . (Color r: 0.4 g: 0.8 b: 1.0) . Color lightCyan . (Color r: 0.599 g: 0.0 b: 0.0) . (Color r: 0.599 g: 0.199 b: 0.0) . (Color r: 0.599 g: 0.4 b: 0.0) . (Color r: 0.599 g: 0.599 b: 0.0) . (Color r: 0.599 g: 0.8 b: 0.0) . (Color r: 0.599 g: 1.0 b: 0.0) . (Color r: 0.599 g: 0.0 b: 0.199) . (Color r: 0.599 g: 0.199 b: 0.199) . (Color r: 0.599 g: 0.4 b: 0.199) . (Color r: 0.599 g: 0.599 b: 0.199) . (Color r: 0.599 g: 0.8 b: 0.199) . (Color r: 0.599 g: 1.0 b: 0.199) . (Color r: 0.599 g: 0.0 b: 0.4) . (Color r: 0.599 g: 0.199 b: 0.4) . (Color r: 0.599 g: 0.4 b: 0.4) . (Color r: 0.599 g: 0.599 b: 0.4) . (Color r: 0.599 g: 0.8 b: 0.4) . (Color r: 0.599 g: 1.0 b: 0.4) . (Color r: 0.599 g: 0.0 b: 0.599) . (Color r: 0.599 g: 0.199 b: 0.599) . (Color r: 0.599 g: 0.4 b: 0.599) . (Color r: 0.599 g: 0.599 b: 0.599) . (Color r: 0.599 g: 0.8 b: 0.599) . (Color r: 0.599 g: 1.0 b: 0.599) . (Color r: 0.599 g: 0.0 b: 0.8) . (Color r: 0.599 g: 0.199 b: 0.8) . (Color r: 0.599 g: 0.4 b: 0.8) . (Color r: 0.599 g: 0.599 b: 0.8) . (Color r: 0.599 g: 0.8 b: 0.8) . (Color r: 0.599 g: 1.0 b: 0.8) . (Color r: 0.599 g: 0.0 b: 1.0) . (Color r: 0.599 g: 0.199 b: 1.0) . (Color r: 0.599 g: 0.4 b: 1.0) . (Color r: 0.599 g: 0.599 b: 1.0) . (Color r: 0.599 g: 0.8 b: 1.0) . (Color r: 0.599 g: 1.0 b: 1.0) . (Color r: 0.8 g: 0.0 b: 0.0) . (Color r: 0.8 g: 0.199 b: 0.0) . (Color r: 0.8 g: 0.4 b: 0.0) . (Color r: 0.8 g: 0.599 b: 0.0) . (Color r: 0.8 g: 0.8 b: 0.0) . (Color r: 0.8 g: 1.0 b: 0.0) . (Color r: 0.8 g: 0.0 b: 0.199) . (Color r: 0.8 g: 0.199 b: 0.199) . (Color r: 0.8 g: 0.4 b: 0.199) . (Color r: 0.8 g: 0.599 b: 0.199) . (Color r: 0.8 g: 0.8 b: 0.199) . (Color r: 0.8 g: 1.0 b: 0.199) . (Color r: 0.8 g: 0.0 b: 0.4) . (Color r: 0.8 g: 0.199 b: 0.4) . (Color r: 0.8 g: 0.4 b: 0.4) . (Color r: 0.8 g: 0.599 b: 0.4) . (Color r: 0.8 g: 0.8 b: 0.4) . (Color r: 0.8 g: 1.0 b: 0.4) . (Color r: 0.8 g: 0.0 b: 0.599) . (Color r: 0.8 g: 0.199 b: 0.599) . (Color r: 0.8 g: 0.4 b: 0.599) . (Color r: 0.8 g: 0.599 b: 0.599) . (Color r: 0.8 g: 0.8 b: 0.599) . (Color r: 0.8 g: 1.0 b: 0.599) . (Color r: 0.8 g: 0.0 b: 0.8) . (Color r: 0.8 g: 0.199 b: 0.8) . (Color r: 0.8 g: 0.4 b: 0.8) . (Color r: 0.8 g: 0.599 b: 0.8) . (Color r: 0.8 g: 0.8 b: 0.8) . (Color r: 0.8 g: 1.0 b: 0.8) . (Color r: 0.8 g: 0.0 b: 1.0) . (Color r: 0.8 g: 0.199 b: 1.0) . (Color r: 0.8 g: 0.4 b: 1.0) . (Color r: 0.8 g: 0.599 b: 1.0) . (Color r: 0.8 g: 0.8 b: 1.0) . Color lightBlue . Color red . (Color r: 1.0 g: 0.199 b: 0.0) . (Color r: 1.0 g: 0.4 b: 0.0) . (Color r: 1.0 g: 0.599 b: 0.0) . (Color r: 1.0 g: 0.8 b: 0.0) . Color yellow . (Color r: 1.0 g: 0.0 b: 0.199) . (Color r: 1.0 g: 0.199 b: 0.199) . (Color r: 1.0 g: 0.4 b: 0.199) . (Color r: 1.0 g: 0.599 b: 0.199) . (Color r: 1.0 g: 0.8 b: 0.199) . (Color r: 1.0 g: 1.0 b: 0.199) . (Color r: 1.0 g: 0.0 b: 0.4) . (Color r: 1.0 g: 0.199 b: 0.4) . (Color r: 1.0 g: 0.4 b: 0.4) . (Color r: 1.0 g: 0.599 b: 0.4) . Color lightOrange . (Color r: 1.0 g: 1.0 b: 0.4) . (Color r: 1.0 g: 0.0 b: 0.599) . (Color r: 1.0 g: 0.199 b: 0.599) . (Color r: 1.0 g: 0.4 b: 0.599) . (Color r: 1.0 g: 0.599 b: 0.599) . (Color r: 1.0 g: 0.8 b: 0.599) . (Color r: 1.0 g: 1.0 b: 0.599) . (Color r: 1.0 g: 0.0 b: 0.8) . (Color r: 1.0 g: 0.199 b: 0.8) . (Color r: 1.0 g: 0.4 b: 0.8) . (Color r: 1.0 g: 0.599 b: 0.8) . Color lightRed . Color lightYellow . Color magenta . (Color r: 1.0 g: 0.199 b: 1.0) . (Color r: 1.0 g: 0.4 b: 1.0) . (Color r: 1.0 g: 0.599 b: 1.0) . Color lightMagenta . Color white})! !

!MCStReader methodsFor: '*CTM-Core-override' stamp: 'SR 1/21/2011 20:34'!
classDefinitionFrom: aPseudoClass
	| tokens traitCompositionString lastIndex classTraitCompositionString rs pos remainingString pragmas |

	"Adding logic for pragma parsing"
	pragmas := OrderedCollection new.
	rs := ReadStream on: aPseudoClass definition.
	[pos := rs position. rs match: '<'] whileTrue: [ | pragmaContent pragmaString pragma |
		pragmaContent := rs upToAll: '>'.
		pragmaString := '<', pragmaContent, '>'.
		pragma := Pragma fromString: pragmaString.
		pragmas add: pragma.
	].
	rs position: pos.
	remainingString := rs upToEnd.
	
	tokens := Scanner new scanTokens: remainingString.
	traitCompositionString := ((ReadStream on: remainingString)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	classTraitCompositionString := ((ReadStream on: aPseudoClass metaClass definition asString)
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	classTraitCompositionString isEmpty ifTrue: [classTraitCompositionString := '{}'].
	lastIndex := tokens size.
	^ MCClassDefinition
		name: (tokens at: 3)
		superclassName: (tokens at: 1)
		traitComposition: traitCompositionString
		classTraitComposition: classTraitCompositionString
		category: (tokens at: lastIndex)
		instVarNames: ((tokens at: lastIndex - 6) findTokens: ' ')
		classVarNames: ((tokens at: lastIndex - 4) findTokens: ' ')
		poolDictionaryNames: ((tokens at: lastIndex - 2) findTokens: ' ')
		classInstVarNames: (self classInstVarNamesFor: aPseudoClass)
		type: (self typeOfSubclass: (tokens at: 2))
		comment: (self commentFor: aPseudoClass)
		commentStamp: (self commentStampFor: aPseudoClass)
		pragmas: pragmas.! !

!MethodReference methodsFor: '*CTM-Core' stamp: 'SR 1/7/2011 14:59'!
isReferenceToGeneratedMethod

	^ self compiledMethod isCTMGeneratedMethod! !

!SequenceableCollection methodsFor: '*CTM-Core' stamp: 'SR 1/7/2011 13:49'!
allCombinationsDo: aBlock

	1 to: self size do: [:i | self combinations: i atATimeDo: aBlock]! !

!SequenceableCollection methodsFor: '*CTM-Core-Override' stamp: 'mig 2/1/2011 18:38'!
concatenation

	| result index |

	result := self anyOne species new: (self inject: 0 into: [:sum :each | sum + each size]).
	index := 0.
	self do: [:each | each do: [:item | result at: (index := index+1) put: item]].
	^ result! !

InstructionClient subclass: #CTMInstVarStoreLocator
	instanceVariableNames: 'foundStore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Examples'!

!CTMInstVarStoreLocator commentStamp: 'mig 1/26/2011 23:35' prior: 0!
A CTMInstVarStoreLocator is finds storing instance variable accesses in a method. Similiar to InstVarRefLocator.!

!CTMInstVarStoreLocator methodsFor: 'instruction decoding' stamp: 'mig 1/26/2011 23:25'!
interpretNextInstructionUsing: aScanner 
	
	foundStore := false.
	aScanner interpretNextInstructionFor: self.
	^ foundStore! !

!CTMInstVarStoreLocator methodsFor: 'instruction decoding' stamp: 'mig 1/26/2011 23:25'!
popIntoReceiverVariable: offset 

	foundStore := true.! !

!CTMInstVarStoreLocator methodsFor: 'instruction decoding' stamp: 'mig 1/26/2011 23:25'!
storeIntoReceiverVariable: offset 

	foundStore := true.! !

!MCClassDefinition class methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 16:18'!
name: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
pragmas: aPragmaArray
	
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: traitCompositionString
					classTraitComposition: classTraitCompositionString
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString
					pragmas: aPragmaArray)! !

!MCClassDefinition methodsFor: '*CTM-Core-override' stamp: 'SR 1/21/2011 16:24'!
= aDefinition
	^(super = aDefinition)
		and: [superclassName = aDefinition superclassName
		and: [self traitCompositionString = aDefinition traitCompositionString
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString
		and: [category = aDefinition category
		and: [type = aDefinition type
		and: [self sortedVariables = aDefinition sortedVariables
		and: [comment = aDefinition comment
		and: [pragmas = aDefinition pragmas]]]]]]]]! !

!MCClassDefinition methodsFor: '*CTM-Core-override' stamp: 'SR 1/21/2011 20:50'!
createClass
	| superClass class composition |
	superClass := superclassName == #nil ifFalse:
					[Smalltalk at: superclassName].
	[class := (ClassBuilder new)
			name: name 
			inEnvironment: superClass environment 
			subclassOf: superClass
			type: type 
			instanceVariableNames: self instanceVariablesString 
			classVariableNames: self classVariablesString 
			poolDictionaries: self sharedPoolsString
			category: category
			unsafe: false
			pragmas: self pragmas.
	] on: Warning, DuplicateVariableError do:[:ex| ex resume].

	"The following is written to support traits unloading"
	composition := Compiler evaluate: (self traitComposition ifNil:['{}']).
	(composition isCollection and:[composition isEmpty and:[class traitComposition isEmpty]]) ifFalse:[
		class setTraitComposition: composition asTraitComposition.
	].

	composition := Compiler evaluate: (self classTraitComposition ifNil:['{}']).
	(composition isCollection and:[composition isEmpty and:[class class traitComposition isEmpty]]) ifFalse:[
		class class setTraitComposition: composition asTraitComposition.
	].

	^class! !

!MCClassDefinition methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 16:12'!
hasPragmas
	^ self pragmas isEmpty not! !

!MCClassDefinition methodsFor: '*CTM-Core' stamp: 'SR 1/21/2011 16:19'!
initializeWithName: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
pragmas: aPragmaArray
	name := nameString asSymbol.
	superclassName := superclassString ifNil: ['nil'] ifNotNil: [superclassString asSymbol].
	traitComposition := traitCompositionString.
	classTraitComposition := classTraitCompositionString.
	category := categoryString.
	name = #CompiledMethod ifTrue: [type := #compiledMethod] ifFalse: [type := typeSymbol].
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray asSortedCollection ofType: MCClassVariableDefinition.
	self addVariables: poolArray asSortedCollection ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition.
	pragmas := aPragmaArray.! !

!MCClassDefinition methodsFor: '*CTM-Core' stamp: 'SR 1/22/2011 12:15'!
pragmas

	^ pragmas ifNil: [ pragmas := OrderedCollection new]! !

!MCClassDefinition methodsFor: '*CTM-Core-override' stamp: 'SR 1/22/2011 12:14'!
printDefinitionOn: stream

		self hasPragmas ifTrue: [
			self pragmas do: [:pragma | pragma printOn: stream. stream cr]].
		
		stream 
			nextPutAll: self superclassName;
			nextPutAll: self kindOfSubclass;
			nextPut: $# ;
			nextPutAll: self className;
			cr; tab.
		self hasTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self traitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString;
			cr; tab;
			nextPutAll: 'classVariableNames: ';
			store: self classVariablesString;
			cr; tab;
			nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString;
			cr; tab;
			nextPutAll: 'category: ';
			store: self category asString! !

!MCClassDefinition methodsFor: '*CTM-Core-override' stamp: 'SR 1/22/2011 12:12'!
storeDataOn: aDataStream
	| instVarSize |
	instVarSize := (self hasTraitComposition or: [ self hasClassTraitComposition ])
		ifTrue: [ self class instSize ]
		ifFalse: [ self class instSize - 2 ].
	self hasPragmas ifFalse: [ instVarSize := instVarSize - 1].
	
	aDataStream
		beginInstance: self class
		size: instVarSize.
	1 to: instVarSize do: [ :index |
		aDataStream nextPut: (self instVarAt: index) ].! !

TestCase subclass: #CTMClassBasedTests
	instanceVariableNames: 'methodManagement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Tests'!

!CTMClassBasedTests methodsFor: 'running' stamp: 'SR 1/15/2011 20:48'!
setUp

	super setUp.
	methodManagement := CTMMethodManagement new.! !

!CTMClassBasedTests methodsFor: 'running' stamp: 'SR 1/15/2011 20:42'!
tearDown

	| class |
	class := Smalltalk at: #CTMTestSubject2 ifAbsent: [].
	class ifNotNil: [ class removeFromSystem ].
	
	^ super tearDown! !

!CTMClassBasedTests methodsFor: 'as yet unclassified' stamp: 'SR 1/15/2011 20:45'!
test01ClassPragmas

	
	| class |
	Compiler evaluate: '<blub>
	Object subclass: #CTMTestSubject2 instanceVariableNames: ''a b c'' classVariableNames: '''' poolDictionaries: '''' category: ''CTM-Tests'''.

	class := Smalltalk at: #CTMTestSubject2.
	self assert: (class pragmas size = 1).
	self assert: (class pragmas first keyword = #blub).! !

!CTMClassBasedTests methodsFor: 'as yet unclassified' stamp: 'SR 1/15/2011 20:52'!
test02ImmutableClassPragma

	
	| class |
	Compiler evaluate: '<immutable>
	Object subclass: #CTMTestSubject2 instanceVariableNames: ''a b c'' classVariableNames: '''' poolDictionaries: '''' category: ''CTM-Tests'''.

	class := Smalltalk at: #CTMTestSubject2.
	
	self should: [
		methodManagement addMethodTo: class in: #testexecution with: 'testMessage

a := 5'.] raise: Error.
	! !

TestCase subclass: #CTMMethodBasedTests
	instanceVariableNames: 'sut'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Tests'!

!CTMMethodBasedTests methodsFor: 'running' stamp: 'SR 1/10/2011 13:42'!
setUp

	super setUp.
	sut := CTMMethodManagement new.
	
	"Clear test class"
	CTMTestSubject methodDictionary removeAll.! !

!CTMMethodBasedTests methodsFor: 'running' stamp: 'SR 1/10/2011 13:34'!
tearDown

	super tearDown! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'SR 1/10/2011 13:43'!
test01AddAndRemoveMethod

	self assert: CTMTestSubject methodDictionary isEmpty.
	
	sut addMethodTo: CTMTestSubject in: #testing with: 'testMessage
	
^ 4'.

	self assert: (CTMTestSubject>>#testMessage) isNil not.
	
	sut removeMethod: CTMTestSubject>>#testMessage.
	
	self assert: CTMTestSubject methodDictionary isEmpty.! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'SR 1/10/2011 13:45'!
test02AddAndRemoveGeneratedMethods

	self assert: CTMTestSubject methodDictionary isEmpty.
	
	sut addMethodTo: CTMTestSubject in: #testing with: 'testMessage
	
^ 4'.
	sut addGeneratedMethodFor: (CTMTestSubject>>#testMessage) with: 'testGeneratedMethod
	
^ 5'.

	self assert: (CTMTestSubject>>#testMessage) isNil not.
	self assert: (CTMTestSubject>>#testGeneratedMethod) isNil not.
	
	sut removeAllGeneratedMethodsFor: CTMTestSubject>>#testMessage.
	
	self assert: (CTMTestSubject>>#testMessage) isNil not.
	self assert: (CTMTestSubject methodDictionary size = 1).! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'SR 1/10/2011 13:50'!
test03AutomaticRemovalOfGeneratedMethods

	self assert: CTMTestSubject methodDictionary isEmpty.
	
	sut addMethodTo: CTMTestSubject in: #testing with: 'testMessage
	
^ 4'.
	sut addGeneratedMethodFor: (CTMTestSubject>>#testMessage) with: 'testGeneratedMethod
	
^ 5'.

	self assert: (CTMTestSubject>>#testMessage) isNil not.
	self assert: (CTMTestSubject>>#testGeneratedMethod) isNil not.
	
	sut removeMethod: CTMTestSubject>>#testMessage.
	
	self assert: CTMTestSubject methodDictionary isEmpty.! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'mig 2/1/2011 11:52'!
test04Shortcuts

	| sut2 |
	self assert: CTMTestSubject methodDictionary isEmpty.
	
	sut addMethodTo: CTMTestSubject in: #testing with: 'testMessage: anArray
<shortcuts: #test count: 2>
^ anArray'.

	self assert: (CTMTestSubject>>#testMessage:) isNil not.
	self assert: (CTMTestSubject>>#test:) isNil not.
	self assert: (CTMTestSubject>>#test:test:) isNil not.
	
	sut2 := CTMTestSubject new.
	self assert: ((sut2 testMessage: {1. 2. 3.}) = {1. 2. 3.}).
	self assert: ((sut2 test: 1) = {1.}).
	self assert: ((sut2 test: 1 test: 2) = {1. 2.}).! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'mig 2/1/2011 11:52'!
test05OptionalParameter

	| sut2 |
	self assert: CTMTestSubject methodDictionary isEmpty.
	
	sut addMethodTo: CTMTestSubject in: #testing with: 'testMessage: param1 test2: param2 test3: param3
<optional: #testMessage: default: 2 optional: #test3: default: 3>
^ {param1. param2. param3.}'.

	self assert: (CTMTestSubject>>#testMessage:test2:test3:) isNil not.
	self assert: (CTMTestSubject>>#test2:) isNil not.
	self assert: (CTMTestSubject>>#testMessage:test2:) isNil not.
	self assert: (CTMTestSubject>>#test2:test3:) isNil not.
	
	sut2 := CTMTestSubject new.
	self assert: ((sut2 testMessage: 5 test2: 6 test3: 7) = {5. 6. 7.}).
	self assert: ((sut2 test2: 5) = {2. 5. 3.}).
	self assert: ((sut2 testMessage: 5 test2: 6) = {5. 6. 3.}).
	self assert: ((sut2 test2: 5 test3: 6) = {2. 5. 6.}).! !

!CTMMethodBasedTests methodsFor: 'as yet unclassified' stamp: 'mig 2/1/2011 11:53'!
test06AssociatedMethods

	| origCount |
	self assert: CTMTestSubject methodDictionary isEmpty.
	
	origCount := CTMTestMacro associatedMethods size.	
	
	sut addMethodTo: CTMTestSubject in: #test with: 'testMessage
<ctmTest>
^ 5'.

	self assert: (CTMTestMacro associatedMethods size = (origCount + 1)).

	sut removeMethod: (CTMTestSubject>>#testMessage).
	
	self assert: (CTMTestMacro associatedMethods size = (origCount)).! !

!AbstractEvent methodsFor: '*CTM-Core' stamp: 'SR 1/15/2011 21:20'!
isCompiled

	^ false! !

!AbstractEvent methodsFor: '*CTM-Core' stamp: 'SR 1/7/2011 13:19'!
oldItem

	^ nil! !

AbstractEvent subclass: #CompiledEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CTM-Core'!

!CompiledEvent class methodsFor: 'accessing' stamp: 'SR 1/15/2011 21:22'!
changeKind

	^ #Compiled! !

!CompiledEvent class methodsFor: 'accessing' stamp: 'SR 1/15/2011 21:22'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind! !

!CompiledEvent methodsFor: 'testing' stamp: 'mig 1/26/2011 23:22'!
isCompiled

	^ true! !

!CompiledEvent methodsFor: 'as yet unclassified' stamp: 'SR 1/15/2011 21:20'!
printEventKindOn: aStream

	aStream nextPutAll: 'Compiled'! !

!SHParserST80 methodsFor: '*CTM-core-override' stamp: 'SR 1/21/2011 11:33'!
parse: isAMethod 
	"Parse the receiver's text. If isAMethod is true
    then treat text as a method, if false as an
    expression with no message pattern"

	self initializeInstanceVariables.
	sourcePosition := 1.
	arguments := Dictionary new.
	temporaries := Dictionary new.
	blockDepth := bracketDepth := 0.
	ranges
		ifNil: [ranges := OrderedCollection new: 100]
		ifNotNil: [ranges reset].
	errorBlock := [^false].
	[
		self scanNext.
		isAMethod ifTrue: [self parseMessagePattern].
		self parsePragmaSequence.
		self parseMethodTemporaries.
		isAMethod ifTrue: [self parsePragmaSequence].
		self parseStatementList.
		currentToken ifNotNil: [self error]
	] ensure:[errorBlock := nil].
	^true! !
CTMSystemPreparation initialize!
